#define INITGUID
#include "../../Library/SDK_Manila_Sense.h"
#include "textfile.h"


map<DWORD,IImagingFactory*> Element::runningThread;
vector<Element*> Element::listElements;
EventHandler* Element::gestEvent=NULL;
bool Element::noError=false;

IImagingFactory *Element::pImgFactory=NULL;//access via getIImagingFactory()

Element::Element(int idElement, int &x,int &y, int type,bool rP){
	gi=GlobalInformations::getInstance();
	typeElement=type;
	paddingSup=0;
	if(!rP){
		x=x*gi->percentOfVGA;//set to correct size
		y=y*gi->percentOfVGA;//set to correct size
	}

	listElements.push_back(this);
	this->idElement=idElement;
	this->owner=NULL;

	position.bottom=y;
	position.top=y;
	position.left=x;
	position.right=x;

	positionAbsolute.bottom=0;
	positionAbsolute.top=0;
	positionAbsolute.left=0;
	positionAbsolute.right=0;

	isActivated=false;
	mouseEvent=MouseCtrl::getInstance();
};


Element::~Element(){
	int nbElement=listElements.size()-1;
	int i=0;
	while(i<nbElement&&listElements[i]!=this){
		i++;
	}
	ASSERT(listElements[i]==this);//if not: Element already deleted...
	for (int j=i;j<nbElement;j++)
	{
		listElements[j]=listElements[j+1];
	}
	listElements[nbElement]=NULL;
	listElements.pop_back();
}

void Element::add(Element* e,int minHeight,int pos,bool mixWithOther){
	MessageBox(NULL,LocalisedString::AddElemError,L"Error : Element.add virtual !",MB_OK | MB_ICONEXCLAMATION);
};

IImagingFactory* Element::getIImagingFactory(){
	DWORD idTmp=GetCurrentThreadId();
	map<DWORD,IImagingFactory*>::iterator it=runningThread.find(idTmp);
	if (it==runningThread.end())
	{
		IImagingFactory* tmp=NULL;
		// Normally you would only call CoInitialize/CoUninitialize
		CoInitializeEx(NULL, COINIT_MULTITHREADED);
		CoCreateInstance (CLSID_ImagingFactory,NULL,
			CLSCTX_INPROC_SERVER,IID_IImagingFactory,
			(void **)&tmp);
		runningThread[idTmp]=tmp;
		return tmp;
	}else
		return runningThread[idTmp];
}

RECT Element::patchImage(DIRECTDRAW_GDI * bkg,wstring *name,int nbPatch,int posX,int posY){

	IImage *pImage = NULL;

	unsigned char decal=name->length();
	if(decal>20)
		decal=name->length()-20;

	getIImagingFactory()->CreateImageFromFile(name->c_str(),&pImage);
	if(pImage==NULL){
		const WCHAR* nameImg=name->c_str()+decal;
		LocalisedString::explainFail(LocalisedString::LoadImageError,nameImg,0);
		IBitmapImage* ibp;
		getIImagingFactory()->CreateNewBitmap(4,4,PixelFormat16bppRGB565,&ibp);
		ibp->QueryInterface(IID_IImage, (void**)&pImage);
		nbPatch=1;
	}

	DIRECTDRAW_GDI *out=bkg;

	ImageInfo im;
	pImage->GetImageInfo(&im);



	RECT posPatch={0,posY,0,posY+im.Height};
	int widthImg=out->width/nbPatch;

	HDC hdc=out->deviceInterface;

	for(int i=0;i<nbPatch;i++){
		posPatch.left=posX+widthImg*i;
		posPatch.right=posPatch.left+im.Width;
		pImage->Draw(hdc, &posPatch, NULL);
	}

	pImage->Release();

	posPatch.left=0;
	posPatch.right=im.Width;
	posPatch.top=0;
	posPatch.bottom=im.Height;

	return posPatch;
};

// Returns true if the conversion took place,
// false if the conversion either unneeded or unavailable
bool Element::ConvertToDIB(HBITMAP& hBitmap)
{
	bool bConverted = false;
	BITMAP stBitmap;
	if (GetObject(hBitmap, sizeof(stBitmap), 
		&stBitmap) && !stBitmap.bmBits)
	{
		// that is a DFB. Now we attempt to create
		// a DIB with the same sizes and pixel format.

		HDC hScreen = GetDC(NULL);
		if (hScreen)
		{
			union {
				BITMAPINFO stBitmapInfo;
				BYTE pReserveSpace[sizeof(BITMAPINFO) 
					+ 0xFF * sizeof(RGBQUAD)];
			};
			ZeroMemory(pReserveSpace, sizeof(pReserveSpace));
			stBitmapInfo.bmiHeader.biSize = sizeof(stBitmapInfo.bmiHeader);
			stBitmapInfo.bmiHeader.biWidth = stBitmap.bmWidth;
			stBitmapInfo.bmiHeader.biHeight = -stBitmap.bmHeight;
			stBitmapInfo.bmiHeader.biPlanes = 1;
			stBitmapInfo.bmiHeader.biBitCount = stBitmap.bmBitsPixel;
			stBitmapInfo.bmiHeader.biCompression = BI_RGB;

			if (stBitmap.bmBitsPixel <= 8)
			{
				stBitmapInfo.bmiHeader.biClrUsed = 
					1 << stBitmap.bmBitsPixel;
				// This image is paletted-managed.

				// Hence we have to synthesize its palette.

			}
			stBitmapInfo.bmiHeader.biClrImportant = 
				stBitmapInfo.bmiHeader.biClrUsed;

			PVOID pBits;
			HBITMAP hDib = CreateDIBSection(hScreen, 
				&stBitmapInfo, DIB_RGB_COLORS, &pBits, NULL, 0);

			if (hDib)
			{
				// ok, we're lucky. Now we have

				// to transfer the image to the DFB.

				HDC hMemSrc = CreateCompatibleDC(NULL);
				if (hMemSrc)
				{
					HGDIOBJ hOldSrc = SelectObject(hMemSrc, hBitmap);
					if (hOldSrc)
					{
						HDC hMemDst = CreateCompatibleDC(NULL);
						if (hMemDst)
						{
							HGDIOBJ hOldDst = SelectObject(hMemDst, hDib);
							if (hOldDst)
							{
								if (stBitmap.bmBitsPixel <= 8)
								{
									// take the DFB's palette and set it to our DIB

									HPALETTE hPalette = 
										(HPALETTE) GetCurrentObject(hMemSrc, OBJ_PAL);
									if (hPalette)
									{
										PALETTEENTRY pPaletteEntries[0x100];
										UINT nEntries = GetPaletteEntries(hPalette, 
											0, stBitmapInfo.bmiHeader.biClrUsed, 
											pPaletteEntries);
										if (nEntries)
										{
											ASSERT(nEntries <= 0x100);
											for (UINT nIndex = 0; nIndex < nEntries; nIndex++)
												pPaletteEntries[nEntries].peFlags = 0;
											VERIFY(SetDIBColorTable(hMemDst, 0, 
												nEntries, (RGBQUAD*) pPaletteEntries) == nEntries);

										}
									}
								}

								// transfer the image using BitBlt function.

								// It will probably end in the

								// call to driver's DrvCopyBits function.

								if (BitBlt(hMemDst, 0, 0, stBitmap.bmWidth, 
									stBitmap.bmHeight, hMemSrc, 0, 0, SRCCOPY))
									bConverted = true; // success


								VERIFY(SelectObject(hMemDst, hOldDst));
							}
							VERIFY(DeleteDC(hMemDst));
						}
						VERIFY(SelectObject(hMemSrc, hOldSrc));
					}
					VERIFY(DeleteDC(hMemSrc));
				}

				if (bConverted)
				{
					VERIFY(DeleteObject(hBitmap)); // it's no longer needed

					hBitmap = hDib;
				}
				else
					VERIFY(DeleteObject(hDib));
			}
			ReleaseDC(NULL, hScreen);
		}
	}
	return bConverted;
}

IImage* Element::getImgFromBuffer(BitmapData* dataBuffer){
	IBitmapImage* pBitmapImg;
	HRESULT hres = getIImagingFactory()->CreateBitmapFromBuffer(dataBuffer, &pBitmapImg);
	if(hres!=S_OK)
		LocalisedString::explainFailIImage(hres,L"getImgFromBuffer CreateBitmapFromBuffer...");

	IImage* pImage=NULL;
	hres = pBitmapImg->QueryInterface(IID_IImage, (void**)&pImage);
	if(hres!=S_OK)
		LocalisedString::explainFailIImage(hres,L"getImgFromBuffer QueryInterface...");
	return pImage;
}




bool Element::saveIcon(HICON myIcon,wstring fileDst){
	ICONINFO icInfo;
	GetIconInfo(myIcon,&icInfo);
	//icInfo.hbmColor
	BITMAP bmIcon,bmMask;
	GetObject(icInfo.hbmColor, sizeof(bmIcon), &bmIcon);
	GetObject(icInfo.hbmMask, sizeof(bmMask), &bmMask);

	if(!ConvertToDIB(icInfo.hbmColor)){
		return false;
	};
	if(!ConvertToDIB(icInfo.hbmMask)){
		return false;
	};
	GetObject(icInfo.hbmColor, sizeof(bmIcon), &bmIcon);
	GetObject(icInfo.hbmMask, sizeof(bmMask), &bmMask);

	if(bmIcon.bmBitsPixel==0){
			return false;
	}
	int iconSize=bmIcon.bmWidth;

	PixelFormat pixelFormat;
	BitmapData bmData = {0};
	//int nStride = bmIcon.bmWidth * bmIcon.bmBitsPixel/8;
	int nStride = bmIcon.bmWidth * 32/8;
	if( (nStride % 4) != 0)
		nStride += 4-(nStride % 4);// *4 !

	bmData.Width = bmIcon.bmWidth;
	bmData.Height = bmIcon.bmHeight;
	bmData.Stride = nStride;
	int widthS;
	switch (bmIcon.bmBitsPixel)
	{/*
	case 1: 
		pixelFormat = PixelFormat1bppIndexed;
		widthS=bmData.Stride;
		break;

	case 4: 
		pixelFormat = PixelFormat4bppIndexed;
		widthS=bmData.Stride;
		break;

	case 8: 
		pixelFormat = PixelFormat8bppIndexed;
		widthS=bmData.Stride;
		break;

	case 16: 
		pixelFormat = PixelFormat16bppARGB1555;
		widthS=bmData.Stride/2;
		break;

	case 24: 
		pixelFormat = PixelFormat24bppRGB;
		widthS=bmData.Stride/4;
		break;*/

	default: 
		pixelFormat = PixelFormat32bppARGB;
		widthS=bmData.Stride/4;
	}

	bmData.PixelFormat=pixelFormat;
	//bmData.PixelFormat=pixelFormat;

	int nbOpaque=0;
	DWORD* buff=new DWORD[widthS*bmData.Height];
	if(bmIcon.bmBitsPixel>24){
		for (unsigned int i=0;i<bmData.Height;i++)
		{
			for(int j=0;j<widthS;j++){
				buff[i*bmData.Width+j]=((DWORD*)bmIcon.bmBits)[i*bmData.Width+j];
			}
		}
	}else{
		if(bmIcon.bmBitsPixel==24){
			int posCpt=0;
			char* destBuff=(char*)buff;
			for (unsigned int i=0;i<bmData.Height;i++)
			{
				for(int j=0;j<widthS;j++){
					destBuff[i*4*bmData.Width+j*4]=((char*)bmIcon.bmBits)[posCpt++];
					destBuff[i*4*bmData.Width+j*4+1]=((char*)bmIcon.bmBits)[posCpt++];
					destBuff[i*4*bmData.Width+j*4+2]=((char*)bmIcon.bmBits)[posCpt++];
					destBuff[i*4*bmData.Width+j*4+3]=0xFF;
				}
			}
			nbOpaque=bmData.Height*bmData.Width;
		}else{
			WORD maskPxl;
			if(bmIcon.bmBitsPixel==16){//555, need to convert:
				WORD pixel;
				for (unsigned int i=0;i<bmData.Height;i++)
				{
					for(unsigned int j=0;j<bmData.Width;j++){
						pixel=((WORD*)bmIcon.bmBits)[i*bmData.Width+j];/*
																	   red=(uchar)((pixel&0x7C00)>>10);
																	   green=(uchar)((pixel&0x03E0)>>5);
																	   blue=(uchar)((pixel&0x001F));*/
						if(bmMask.bmBitsPixel==1){
							uchar tmp1=i;
							uchar tmp2=j/8;
							uchar pixTmp=((uchar*)bmMask.bmBits)[tmp1*bmMask.bmWidthBytes+tmp2];
							maskPxl=pixTmp&(uchar)pow(2,7-(j%8));
							if (maskPxl==0)
								buff[i*bmData.Width+j]=RGBA555to32(0,pixel);
							else{
								buff[i*bmData.Width+j]=RGBA555to32(0xFF000000,pixel);
								nbOpaque++;
							}
						}else{
							buff[i*bmData.Width+j]=RGBA555to32(0xFF000000,pixel);
							nbOpaque=200;//set to a value (no mask)
						}
					}
				}
			}else{//get palette definition:
				HDC           hMemDC=NULL;
				HBITMAP       hOldBitmap=NULL;
				RGBQUAD       rgb[256];

				// Create a memory DC and select the DIBSection into it
				hMemDC = CreateCompatibleDC( NULL );
				hOldBitmap = (HBITMAP)SelectObject( hMemDC, icInfo.hbmColor );
				// Get the DIBSection's color table
				GetDIBColorTable( hMemDC, 0, 256, rgb );
				SelectObject( hMemDC, hOldBitmap );
				DeleteDC( hMemDC );

				uchar pixel;
				for (unsigned int i=0;i<bmData.Height;i++)
				{
					for(unsigned int j=0;j<bmData.Width;j++){
						pixel=((uchar*)bmIcon.bmBits)[i*bmData.Width+j];
						if(bmMask.bmBitsPixel==1){
							uchar tmp1=i;
							uchar tmp2=j/8;
							uchar pixTmp=((uchar*)bmMask.bmBits)[tmp1*bmMask.bmWidthBytes+tmp2];
							maskPxl=pixTmp&(uchar)pow(2,7-(j%8));
							if (maskPxl==0)
								buff[i*bmData.Width+j]=(DWORD)0x00FFFFFF&((DWORD)rgb[pixel].rgbReserved)<<24|((DWORD)rgb[pixel].rgbRed)<<16|((DWORD)rgb[pixel].rgbGreen)<<8|rgb[pixel].rgbBlue;
							else{
								buff[i*bmData.Width+j]=(DWORD)0xFF000000|((DWORD)rgb[pixel].rgbReserved)<<24|((DWORD)rgb[pixel].rgbRed)<<16|((DWORD)rgb[pixel].rgbGreen)<<8|rgb[pixel].rgbBlue;
								nbOpaque++;
							}
						}else{
							buff[i*bmData.Width+j]=(DWORD)0xFF000000|((DWORD)rgb[pixel].rgbReserved)<<24|((DWORD)rgb[pixel].rgbRed)<<16|((DWORD)rgb[pixel].rgbGreen)<<8|rgb[pixel].rgbBlue;
							nbOpaque=200;
						}
					}
				}
			}
		}
	}

	if(nbOpaque==0){
		for (int i=0;i<bmData.Height;i++)
		{
			for(int j=0;j<bmData.Width;j++){
				if (buff[i*bmData.Width+j]!=0)
					buff[i*bmData.Width+j]=0xFF000000|buff[i*bmData.Width+j];
			}
		}
	}

	if(nbOpaque==bmData.Height*bmData.Width){
		for (int i=0;i<bmData.Height;i++)
		{
			for(int j=0;j<bmData.Width;j++){
				if (buff[i*bmData.Width+j]==0xFF000000)
					buff[i*bmData.Width+j]=0;
			}
		}
	}
	bmData.Scan0 = buff;

	HRESULT hres = S_OK;
	IImage* pImage=getImgFromBuffer(&bmData);
	ImageInfo imIn;
	hres = pImage->GetImageInfo(&imIn);

	//find the PNG codec :
	UINT            count=0;
	ImageCodecInfo* encoders=NULL;
	getIImagingFactory()->GetInstalledEncoders(&count,&encoders);
	int cpt=0;
	while(cpt<count&&encoders[cpt].FormatID!=ImageFormatPNG)
		cpt++;
	if(cpt<count){
		//we found it!
		IImageEncoder *myEncoder=NULL;
		HRESULT hr;
		if (FAILED(hr=getIImagingFactory()->CreateImageEncoderToFile(&encoders[cpt].Clsid,fileDst.c_str(), &myEncoder)))
		{
			LocalisedString::explainFailIImage(hr,L"CreateImageEncoderToFile");
			goto exit;
		}
		IImageSink* sinkOfImg=NULL;
		if (FAILED(hr=myEncoder->GetEncodeSink(&sinkOfImg)))
		{
			LocalisedString::explainFailIImage(hr,L"GetEncodeSink");
			goto exit;
		}
		if (FAILED(hr=sinkOfImg->BeginSink(&imIn,NULL)))
		{
			LocalisedString::explainFailIImage(hr,L"BeginSink");
			goto exit;
		}

		if (FAILED(hr=pImage->PushIntoSink(sinkOfImg)))
		{
			LocalisedString::explainFailIImage(hr,L"PushIntoSink");
			goto exit;
		}

		myEncoder->TerminateEncoder();
		myEncoder->Release();
		sinkOfImg->Release();
		myEncoder->Release();
	}
	CoTaskMemFree(&encoders);


	pImage->Release();
	//pBitmapImg->Release();
	delete []buff;
	return true;

exit:

	pImage->Release();
	//pBitmapImg->Release();
	delete []buff;
	return false;
};

HICON Element::loadIcon(wstring fileDst,wstring link){
	HICON hIconLarge=0, hIconSmall=0,myIcon=0;

	myIcon=ExtractIconEx(fileDst.c_str(), 0,&hIconLarge, &hIconSmall,1);
	if(myIcon==NULL){
		//look into the link :
		TextFile tf;
		if(tf.Open(link.c_str(),TF_READ)>=0){
			WCHAR *myLine=NULL;
			int nbChar=tf.ReadLine(NULL,&myLine);
			wstring line=myLine;
			int posInter=line.find(L"?");
			if(posInter!=wstring::npos){
				line=line.substr(posInter+1);
				posInter=line.find(L",");
				if(posInter!=wstring::npos){
					wstring indexRes=line.substr(posInter+1);
					if(line.find(L"ceshell")!=wstring::npos)
						line=L"\\windows\\shellres.dll";//trick...
					else
						line=line.substr(0,posInter);
					wstringstream outNB;
					outNB<<indexRes;
					int id;
					outNB>>id;
					myIcon=ExtractIconEx(line.c_str(), id,&hIconLarge, &hIconSmall,1);
					if(myIcon==NULL){
						wstring lineTmp=line.substr(0,line.find_last_of(L'.'));
						lineTmp=lineTmp+L".192";
						line=lineTmp+line.substr(line.find_last_of(L'.'));
						myIcon=ExtractIconEx(line.c_str(), id,&hIconLarge, &hIconSmall,1);
					}
				}
			}
		}
	}
	if(myIcon==NULL){
		return 0;
	}
	if(hIconLarge==myIcon&&hIconSmall!=0)
		DestroyIcon(hIconSmall);
	if(hIconSmall==myIcon&&hIconLarge!=0)
		DestroyIcon(hIconLarge);
	
	return myIcon;
}


IImage * Element::loadIImage(wstring *name){
	IImage *pImage = NULL;

	unsigned char decal=name->length();
	if(decal>20)
		decal=name->length()-20;
	else
		decal=0;

	getIImagingFactory()->CreateImageFromFile(name->c_str(),&pImage);
	if(pImage==NULL){//create a fake image...
		if(noError)
			return NULL;
		const WCHAR* nameImg=name->c_str()+decal;
		LocalisedString::explainFail(LocalisedString::LoadImageError,nameImg,0);
		IBitmapImage* ibp;
		getIImagingFactory()->CreateNewBitmap(4,4,PixelFormat16bppRGB565,&ibp);
		ibp->QueryInterface(IID_IImage, (void**)&pImage);
		return pImage;
	}
	return pImage;
}

void Element::resizeIImage(wstring *name){
	IImage *pImage = NULL;

	unsigned char decal=name->length();
	if(decal>20)
		decal=name->length()-20;
	else
		decal=0;

	getIImagingFactory()->CreateImageFromFile(name->c_str(),&pImage);
	if(pImage==NULL){
		return;
	}
	ImageInfo imIn;
	pImage->GetImageInfo(&imIn);
	IBitmapImage* newImgResized=NULL;
	GlobalInformations* giGlobal=GlobalInformations::getInstance();
	imIn.Width*=giGlobal->percentOfVGA;
	imIn.Height*=giGlobal->percentOfVGA;
	getIImagingFactory()->CreateBitmapFromImage(pImage,imIn.Width,imIn.Height,
		imIn.PixelFormat,InterpolationHintBicubic,&newImgResized);
	pImage->Release();
	if(newImgResized!=NULL){

		IImage *pImage1 = NULL;
		newImgResized->QueryInterface(IID_IImage, (void**)&pImage1);
		pImage1->GetImageInfo(&imIn);

		//find the PNG codec :
		UINT            count=0;
		ImageCodecInfo* encoders=NULL;
		getIImagingFactory()->GetInstalledEncoders(&count,&encoders);
		unsigned int cpt=0;
		while(cpt<count&&encoders[cpt].FormatID!=ImageFormatPNG)
			cpt++;
		if(cpt<count){
			//we found it!
			IImageEncoder *myEncoder=NULL;
			HRESULT hr;
			if (FAILED(hr=getIImagingFactory()->CreateImageEncoderToFile(&encoders[cpt].Clsid,name->c_str(), &myEncoder)))
			{
				LocalisedString::explainFailIImage(hr,L"CreateImageEncoderToFile");
				return;
			}
			IImageSink* sinkOfImg=NULL;
			if (FAILED(hr=myEncoder->GetEncodeSink(&sinkOfImg)))
			{
				LocalisedString::explainFailIImage(hr,L"GetEncodeSink");
				return;
			}
			if (FAILED(hr=sinkOfImg->BeginSink(&imIn,NULL)))
			{
				LocalisedString::explainFailIImage(hr,L"BeginSink");
				return;
			}

			if (FAILED(hr=pImage1->PushIntoSink(sinkOfImg)))
			{
				LocalisedString::explainFailIImage(hr,L"PushIntoSink");
				return;
			}

			myEncoder->TerminateEncoder();
			sinkOfImg->Release();
			myEncoder->Release();
		}
		CoTaskMemFree(&encoders);

		newImgResized->Release();
	}
}

DIRECTDRAW_GDI * Element::fromIImageToGDI( IImage** src,bool force32bits/*=false*/ )
{
	IImage* pImage=*src;
	DIRECTDRAW_GDI *out=new DIRECTDRAW_GDI;
	out->directDrawSurf=NULL;

	ImageInfo imageInfo;
	pImage->GetImageInfo( &imageInfo );
	if(force32bits){
		imageInfo.PixelFormat=PixelFormat32bppPARGB;
	}
	IBitmapImage* pBitmapImage = NULL;
	getIImagingFactory()->CreateBitmapFromImage( pImage, 0, 0,imageInfo.PixelFormat, InterpolationHintNearestNeighbor , &pBitmapImage );
	RECT rect={ 0, 0, imageInfo.Width, imageInfo.Height };
	BitmapData bitmapData;
	pBitmapImage->LockBits(&rect, ImageLockModeRead, imageInfo.PixelFormat, &bitmapData );
	out->bitmap = CreateBitmap( imageInfo.Width, imageInfo.Height, 1, GetPixelFormatSize( imageInfo.PixelFormat ), bitmapData.Scan0 );

	BITMAP bm;
	GetObject(out->bitmap, sizeof(bm), &bm);
	out->width=bm.bmWidth;
	out->height=bm.bmHeight;
	out->nbBPP=bm.bmBitsPixel;

	//
	// Select bitmap into a memoryDC so we can use it.
	//
	out->deviceInterface = CreateCompatibleDC(NULL);
	if (!out->deviceInterface) {
		LocalisedString::explainFail(LocalisedString::szGetDC,L"Section: CreateCompatibleDC(NULL);");
		return NULL;
	}
	out->oldObj=SelectObject(out->deviceInterface, out->bitmap);
	pImage->Release();
	pBitmapImage->Release();
	*src=NULL;
	return out;
}

DIRECTDRAW_GDI * Element::loadImage(wstring *name,bool rescall,bool transparent){
#ifdef DRAWING_GDI
	DIRECTDRAW_GDI *out;
/*
	out->bitmap=SHLoadImageFile(name->c_str());
	if(out->bitmap==NULL){
		unsigned char decal=name->length();
		if(decal>20)
			decal=name->length()-20;
		else
			decal=0;
		LocalisedString::explainFail(LocalisedString::LoadImageError,name->c_str()+decal);
		return NULL;
	}
	*/

	// Get size of the bitmap


	IImage* pImage=loadIImage( name);
	if(pImage==NULL)
		return NULL;
	bool force32b=name->find(L".png")!=wstring::npos;
	out=fromIImageToGDI(&pImage,force32b);

	if(rescall){
		int oldH=out->height;
		int oldW=out->width;
		GlobalInformations* giGlobal=GlobalInformations::getInstance();
		out->width=out->width*giGlobal->percentOfVGA;
		out->height=out->height*giGlobal->percentOfVGA;
		HDC cDC = CreateCompatibleDC(NULL);
		HBITMAP hbmp = CreateCompatibleBitmap( out->deviceInterface, out->width, out->height );
		HGDIOBJ oldObj=SelectObject( cDC, hbmp );
		if (StretchBlt(cDC,
			0,0,out->width,out->height,
			out->deviceInterface, 
			0,0,oldW,oldH,SRCCOPY)==0){
				LocalisedString::explainFailWinAPI(L"StretchBlt error");
		}
		SelectObject(out->deviceInterface, out->oldObj);
		DeleteObject (out->bitmap);
		DeleteDC(out->deviceInterface);
		out->oldObj=oldObj;
		out->bitmap=hbmp;
		out->deviceInterface=cDC;
	}

	return out;
#endif

#ifdef DRAWING_DDRAW
	IImage *pImage = NULL;
	DIRECTDRAW_GDI *out=new DIRECTDRAW_GDI;

	unsigned char decal=name->length();
	if(decal>20)
		decal=name->length()-20;
	else
		decal=0;

	getIImagingFactory()->CreateImageFromFile(name->c_str(),&pImage);
	if(pImage==NULL){
		const WCHAR* nameImg=name->c_str()+decal;
		LocalisedString::explainFail(LocalisedString::LoadImageError,nameImg,0);
		DDSURFACEDESC ddsd;
		memset(&ddsd, 0, sizeof(ddsd));
		ddsd.dwSize = sizeof(ddsd);
		ddsd.dwFlags = DDSD_WIDTH | DDSD_HEIGHT;
		ddsd.dwWidth = 4;
		ddsd.dwHeight = 4;
		HRESULT hRet = (*gi->ldDDraw)->CreateSurface(&ddsd, &out->directDrawSurf, NULL );
		if (hRet != DD_OK)
			LocalisedString::explainFailDirectDraw(hRet,L"loadImage CreateSurface");
		out->directDrawSurf->GetDC(&out->deviceInterface);
		return out;
	}

	if(rescall){
		ImageInfo imIn;
		pImage->GetImageInfo(&imIn);
		IBitmapImage* newImgResized=NULL;
		imIn.Width*=gi->percentOfVGA;
		imIn.Height*=gi->percentOfVGA;
		getIImagingFactory()->CreateBitmapFromImage(pImage,imIn.Width,imIn.Height,
			imIn.PixelFormat,InterpolationHintBicubic,&newImgResized);
		if(newImgResized!=NULL){
			pImage->Release();
			pImage = NULL;
			newImgResized->QueryInterface(IID_IImage, (void**)&pImage);
		}
	}

	ImageInfo im;
	pImage->GetImageInfo(&im);

	DDSURFACEDESC ddsd;
	memset(&ddsd, 0, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	ddsd.dwFlags = DDSD_WIDTH | DDSD_HEIGHT;
	ddsd.dwWidth = im.Width;
	ddsd.dwHeight = im.Height;
/*
	if(transparent){
		ddsd.dwFlags|=DDSD_PIXELFORMAT;
		ddsd.ddpfPixelFormat.dwSize = sizeof(ddsd.ddpfPixelFormat);
		switch (im.PixelFormat)
		{
		case PixelFormat32bppPARGB:
			ddsd.ddpfPixelFormat.dwFlags=DDPF_ALPHAPREMULT;
		case PixelFormat32bppARGB:
			ddsd.ddpfPixelFormat.dwFlags=ddsd.ddpfPixelFormat.dwFlags|DDPF_ALPHAPIXELS|DDPF_RGB;
			ddsd.ddpfPixelFormat.dwAlphaBitDepth=8;
			ddsd.ddpfPixelFormat.dwRGBBitCount=32;//with alpha !
			ddsd.ddpfPixelFormat.dwRGBAlphaBitMask	=0xFF000000;
			ddsd.ddpfPixelFormat.dwRBitMask			=0x00FF0000;
			ddsd.ddpfPixelFormat.dwGBitMask			=0x0000FF00;
			ddsd.ddpfPixelFormat.dwBBitMask			=0x000000FF;
			break;
		default:
			ddsd.ddpfPixelFormat.dwFlags=DDPF_RGB;
			ddsd.ddpfPixelFormat.dwRGBBitCount=16;
			ddsd.ddpfPixelFormat.dwRBitMask=0x1F<<11;
			ddsd.ddpfPixelFormat.dwGBitMask=0x3F<<5;
			ddsd.ddpfPixelFormat.dwBBitMask=0x1F;
		}
	}*/

	HRESULT hRet = (*gi->ldDDraw)->CreateSurface(&ddsd, &out->directDrawSurface, NULL );
	if (hRet != DD_OK)
		LocalisedString::explainFailDirectDraw(hRet,L"loadImage CreateSurface");
	hRet = out->directDrawSurface->GetDC(&out->deviceInterface);
	if (hRet != DD_OK)
		LocalisedString::explainFailDirectDraw(hRet,L"loadImage GetDC");

	//set transparent background :
	DDBLTFX ddbltfx;
	memset(&ddbltfx, 0, sizeof(ddbltfx));
	ddbltfx.dwSize = sizeof(ddbltfx);
	ddbltfx.dwFillColor = BACKGROUND_COLOR_MANILA;

	//hRet = gi->window->BltFast(out, NULL, NULL, NULL, DDBLT_COLORFILL|DDBLT_WAITNOTBUSY, &ddbltfx );
	hRet = gi->window->BltFast(out, NULL, NULL, NULL, DDBLT_COLORFILL|DDBLT_WAITNOTBUSY, &ddbltfx);
	if (hRet != DD_OK)
		LocalisedString::explainFailDirectDraw(hRet,L"loadImage init Blt");

	DDCOLORKEY ddck;
	memset(&ddck, 0, sizeof(ddck));
	ddck.dwColorSpaceHighValue = ddbltfx.dwFillColor;
	ddck.dwColorSpaceLowValue = ddck.dwColorSpaceHighValue;
	hRet = out->directDrawSurface->SetColorKey(DDCKEY_SRCBLT,&ddck);
	if (hRet != DD_OK)
		LocalisedString::explainFailDirectDraw(hRet,L"out SetColorKey");

	RECT sizeImg={0,0,im.Width,im.Height};
	HDC hdc;
	hdc=out->deviceInterface;

	HRESULT h=pImage->Draw(hdc, &sizeImg, NULL);
	if(h!=S_OK){
		const WCHAR* nameImg=name->c_str()+decal;
		LocalisedString::explainFailIImage(h,nameImg);
	}

	pImage->Release();
#endif
	return out;

};

void Element::setMustRedraw(){
	if(positionAbsolute.right==positionAbsolute.bottom&&
		positionAbsolute.bottom==0)
		return;//OUT of screen, no needs for redraw !
	positionAbsolute.right=positionAbsolute.left+this->getWidth();
	if(positionAbsolute.right>GetSystemMetrics(SM_CXSCREEN))
		positionAbsolute.right=GetSystemMetrics(SM_CXSCREEN);
	if(positionAbsolute.left<0)
		positionAbsolute.left=0;
	positionAbsolute.bottom=positionAbsolute.top+this->getHeight();
	if(positionAbsolute.bottom>GetSystemMetrics(SM_CYSCREEN))
		positionAbsolute.bottom=GetSystemMetrics(SM_CYSCREEN);
	if(positionAbsolute.top<gi->rs.top)
		positionAbsolute.top=gi->rs.top;
	if(positionAbsolute.bottom-positionAbsolute.top>0&&
		positionAbsolute.right-positionAbsolute.left>0){
		gi->window->addToRedraw(positionAbsolute,true);
	}
}


bool Element::run(bool isRightClick){
	if(isRightClick){
		getGestEvent()->messageTreatment(this,EVENT_RIGHT_CLICK);
		//gestEvent->messageProc(this,EVENT_RIGHT_CLICK);
	}else{
		getGestEvent()->messageTreatment(this,EVENT_CLICK);
		//gestEvent->messageProc(this,EVENT_CLICK);
	}
	return false;
};

void Element::sendMsg(int wParam,void* lParam){
	gestEvent->messageProc(lParam,wParam);
};

Element* Element::getElementByID(int id,int typeExpected){
	int nbElement=listElements.size();
	for(int i=0;i<nbElement;i++){
		if (listElements[i]->idElement==id){
			if(typeExpected<=0)
				return listElements[i];
			else{
				if (typeExpected==listElements[i]->typeElement)
				{
					return listElements[i];
				}
			}
		}
	}
	return NULL;
};

void Element::redraw(){
	int nbElement=listElements.size();
	for(int i=0;i<nbElement;i++){
		listElements[i]->setMustRedraw();
	}
	//Sleep(10);//wait for previous drawing...
};

void Element::reloadAllSurface(){
	DWORD dwWidth = GetSystemMetrics(SM_CXSCREEN);
	DWORD dwHeight = GetSystemMetrics(SM_CYSCREEN);
	DDSURFACEDESC dd;
	ZeroMemory(&dd,sizeof(dd));
	dd.dwHeight=dwHeight;
	dd.dwWidth=dwWidth;
	DDBLTFX ddbl;
	ddbl.dwFillColor=DirectDrawWarp::background_color;

	GlobalInformations* giGlobal=GlobalInformations::getInstance();

	if(Section::scrollBar!=NULL){
		if(Section::scrollBar->directDrawSurf!=NULL){
			Section::scrollBar->directDrawSurf->ReleaseDC(Section::scrollBar->deviceInterface);
			Section::scrollBar->directDrawSurf->Release();
		}else{
			if(Section::scrollBar->deviceInterface!=NULL){
				if(DeleteDC(Section::scrollBar->deviceInterface)==0)
					LocalisedString::explainFailWinAPI(L"DeleteObject");
				if(DeleteObject (Section::scrollBar->bitmap)==0)
					LocalisedString::explainFailWinAPI(L"DeleteObject");
			}
		}
		delete Section::scrollBar;
		Section::scrollBar=NULL;//will be recreated in Section
	}
	(*(giGlobal->ldDDraw))->RestoreAllSurfaces();

	if(giGlobal->backBuffer!=NULL){
		DirectDrawWarp::deleteSurf(&giGlobal->backBuffer);

		giGlobal->backBuffer=new DIRECTDRAW_GDI;
		DirectDrawWarp::createSurf(&dd,giGlobal->backBuffer);
		giGlobal->window->BltFast(giGlobal->backBuffer,NULL,NULL,NULL,DDBLT_COLORFILL,&ddbl);
	}
	if(giGlobal->background!=NULL){
		DirectDrawWarp::deleteSurf(&giGlobal->background);

		giGlobal->background=new DIRECTDRAW_GDI;
		DirectDrawWarp::createSurf(&dd,giGlobal->background);
		giGlobal->window->BltFast(giGlobal->background,NULL,NULL,NULL,DDBLT_COLORFILL,&ddbl);
		giGlobal->window->setBackgroundImg(NULL);//update background
	}
	if(giGlobal->primaryBuffer!=NULL){
		//gi->primaryBuffer=DirectDrawWarp::rotateGDI90(gi->primaryBuffer);
		giGlobal->primaryBuffer->width=dwWidth;
		giGlobal->primaryBuffer->height=dwHeight;
	}

	giGlobal->rcUpdateRegion=giGlobal->rs;
	int nbElement=listElements.size();
	for(int i=0;i<nbElement;i++){
		listElements[i]->loadSurf();
	}
};

void Element::sizeChanged(Element* src,int width,int height){
	setWidth(width);
	setHeight(height);
}


DIRECTDRAW_GDI* Element::CreateBitmapMask(DIRECTDRAW_GDI *hbmColour, COLORREF crTransparent)
{
	DIRECTDRAW_GDI* out=new DIRECTDRAW_GDI;
	out->directDrawSurf=NULL;


	out->bitmap = CreateBitmap(hbmColour->width,hbmColour->height, 1, 1, NULL);

	// Get some HDCs that are compatible with the display driver

	out->deviceInterface = CreateCompatibleDC(0);

	out->oldObj=SelectObject(out->deviceInterface, out->bitmap);

	// Set the background colour of the colour image to the colour
	// you want to be transparent.
	SetBkColor(hbmColour->deviceInterface, crTransparent);

	// Copy the bits from the colour image to the B+W mask... everything
	// with the background colour ends up white while everythig else ends up
	// black...Just what we wanted.

	if(BitBlt(out->deviceInterface, 0, 0, hbmColour->width,hbmColour->height, hbmColour->deviceInterface, 0, 0, SRCCOPY)==0){
		LocalisedString::explainFailWinAPI(L"CreateBitmapMask BitBlt");
	}

	return out;
}
