#include "../../Library/SDK_Manila_Sense.h"

MouseCtrl*			MouseCtrl::instance=NULL;
CRITICAL_SECTION	MouseCtrl::mouseUpdate;
GlobalInformations* MouseCtrl::gi=NULL;

MouseCtrl::MouseCtrl(void)
{
	gi=GlobalInformations::getInstance();
	//Mouse interface :
	mouseOldXPos=0;
	mouseOldYPos=0;
	mouseNewXPos=0;
	mouseNewYPos=0;
	mouseClickXPos=0;
	mouseClickYPos=0;
	lastMouseXPos=0;
	lastMouseYPos=0;

	somethingAppend=true;
	scrolling=false;
	clickSelected=false;
	mouseDown=false;
	splClick=false;
	dblClick=false;
	hasInfoAboutClick=false;
	clickPosGet=false;


	lastTickMouse=0;
	startScrolling=0;
	lastScroll=0;
	for(int i=0;i<NB_MOUSE_HISTORY;i++){
		userYVel[i]=0;
		userXVel[i]=0;
	}
	this->idPos=0;
	
	InitializeCriticalSection(&MouseCtrl::mouseUpdate);//MouseCtrl is singleton, so initilised once...
	gestMouse=NULL;
}

MouseCtrl::~MouseCtrl(void)
{
	instance=NULL;//we are destroyed (singleton)
	gi=NULL;
}

MouseCtrl* MouseCtrl::getInstance(){
	if(instance==NULL)
		instance=new MouseCtrl();
	return instance;
};

void MouseCtrl::rightBtnDown(int x,int y){
	EnterCriticalSection(&MouseCtrl::mouseUpdate);

	rightClick=true;
	slideLeft=false;
	dblClick=false;
	clickSelected=false;
	mouseDown=false;
	clickPosGet=false;
	mouseClickXPos=x;
	mouseClickYPos=y;
	mouseNewXPos=x;
	mouseNewYPos=y;
	mouseOldXPos=x;
	mouseOldYPos=y;
	lastMouseXPos=x;
	lastMouseYPos=y;

	somethingAppend=true;
	cantSlideLeft=false;
	stopScroll();

	userXVel[0]=0;
	userYVel[0]=0;
	idPos=0;
	LeaveCriticalSection(&MouseCtrl::mouseUpdate);
}


void MouseCtrl::leftBtnDown(int x,int y){
	EnterCriticalSection(&MouseCtrl::mouseUpdate);
	lastTickMouse=GetTickCount();
	if(!splClick)
		clickEvent=lastTickMouse;

	rightClick=false;
	slideLeft=false;
	dblClick=false;
	clickSelected=true;
	mouseDown=true;
	clickPosGet=false;
	mouseClickXPos=x;
	mouseClickYPos=y;
	mouseNewXPos=x;
	mouseNewYPos=y;
	mouseOldXPos=x;
	mouseOldYPos=y;
	lastMouseXPos=x;
	lastMouseYPos=y;

	somethingAppend=true;
	cantSlideLeft=false;
	stopScroll();

	userXVel[0]=0;
	userYVel[0]=0;
	idPos=0;
	LeaveCriticalSection(&MouseCtrl::mouseUpdate);

}

void MouseCtrl::leftBtnUp(int x,int y){
	EnterCriticalSection(&MouseCtrl::mouseUpdate);
	//this position don't need to be buffered...

	somethingAppend=true;
	mouseDown=false;
	DWORD tick=GetTickCount();
	double timeBetweenPosition = (double)(tick - lastTickMouse);
	
	if (clickSelected){
		if(splClick&&((double)(tick - clickEvent)<DBL_CLICK_LENGTH)){
			splClick=false;
			dblClick=true;
		}else{
			clickEvent=tick;
			hasInfoAboutClick=false;
			splClick=true;
		}
		clickSelected=false;
	}
	if(idPos>2){
		unsigned char posHistory=(idPos-2)%NB_MOUSE_HISTORY;
		unsigned char posHistory1=(idPos-1)%NB_MOUSE_HISTORY;
		if(abs(this->userYVel[posHistory1]+this->userYVel[posHistory])>10&&
			timeBetweenPosition<300){
			// Create a timer to show scrolling
			SetTimer(gi->m_hWnd, 1, MS_PER_FRAME, NULL);
			scrolling=true;
			startScrolling=GetTickCount();
			lastScroll=startScrolling;
		}
	}else
		idPos++;

	LeaveCriticalSection(&MouseCtrl::mouseUpdate);
};
void MouseCtrl::stopScroll(){
	if(scrolling){
		scrolling=false;
		// stop timer (used for scroll...)
		KillTimer(gi->m_hWnd, 1);
	}
};

void MouseCtrl::leftBtnMove(int x,int y){		//mouse drag :
	EnterCriticalSection(&MouseCtrl::mouseUpdate);
	rightClick=false;
	DWORD tick=GetTickCount();
	double timeBetweenPosition = (double)(tick - lastTickMouse);
	if(timeBetweenPosition<3){
		LeaveCriticalSection(&MouseCtrl::mouseUpdate);
		return;
	}
	if (clickSelected){
		if(abs(mouseClickXPos-x)>30||abs(mouseClickYPos-y)>30){
			clickSelected=false;
		}
	}
	somethingAppend=true;
	stopScroll();
	//scrolling event X or Y ?
	if(!cantSlideLeft&&abs(mouseClickYPos-y)<30){//slide X enough fast, long and Y not changed
		if(abs(mouseClickXPos-x)>150)
			slideLeft=true;
	}else{
		cantSlideLeft=true;
		slideLeft=false;
	}
	
	mouseOldXPos=mouseNewXPos;
	mouseOldYPos=mouseNewYPos;

	mouseNewXPos=x;
	mouseNewYPos=y;
	
	lastTickMouse=tick;
	unsigned char posHistory=(idPos)%NB_MOUSE_HISTORY;
	this->userXVel[posHistory]=100.0*(double)(mouseNewXPos-mouseOldXPos)/timeBetweenPosition;
	this->userYVel[posHistory]=100.0*(double)(mouseNewYPos-mouseOldYPos)/timeBetweenPosition;
	if(this->userYVel[posHistory]>500){
		this->userYVel[posHistory]=500;
	}
	if(this->userYVel[posHistory]<-500){
		this->userYVel[posHistory]=-500;
	}

	idPos++;
	LeaveCriticalSection(&MouseCtrl::mouseUpdate);

};

bool MouseCtrl::setMouseAnalyse(Element* e){
	EnterCriticalSection(&MouseCtrl::mouseUpdate);
	if(gestMouse==NULL){
		gestMouse=e;
		LeaveCriticalSection(&MouseCtrl::mouseUpdate);
		return true;
	}
	LeaveCriticalSection(&MouseCtrl::mouseUpdate);
	return false;
}

Element* MouseCtrl::changeMouseAnalyse(Element* e){
	EnterCriticalSection(&MouseCtrl::mouseUpdate);

	if(e==NULL){
		LeaveCriticalSection(&MouseCtrl::mouseUpdate);
		return NULL;
	}
	//reinit mouse :
	slideLeft=false;
	somethingAppend=true;
	stopScroll();
	clickSelected=false;//true;//a click is the source...
	splClick=false;
	dblClick=false;
	hasInfoAboutClick=false;

	Element* mouseAnalysePrev=gestMouse;
	gestMouse=e;
	return mouseAnalysePrev;
};


int MouseCtrl::getMouseDY(){
	int out=0;
	EnterCriticalSection(&MouseCtrl::mouseUpdate);
	DWORD tick= GetTickCount();
	double timeFrame=(double)(tick-lastScroll);
	if(timeFrame<2){
		LeaveCriticalSection(&MouseCtrl::mouseUpdate);
		return 0;
	}

	if(scrolling){
		if(idPos<2){
			stopScroll();
			LeaveCriticalSection(&MouseCtrl::mouseUpdate);
			return 0;
		}

		double timeAnim	=(double)(tick-startScrolling);
		lastScroll=tick;

		double speedY=(2.0*userYVel[(idPos-1)%NB_MOUSE_HISTORY]+userYVel[(idPos-2)%NB_MOUSE_HISTORY])/3.0;

		speedY=timeFrame*speedY/100.0;
		if(speedY>120)
			speedY=120;
		if(speedY<-120)
			speedY=-120;
		bool up=speedY<0;

		if(up)
			timeAnim=-timeAnim;

		double test=speedY-(timeAnim/VEL_INERTIE);
		if(abs((int)test)<1||abs((int)test)>abs((int)speedY)||(up&&test>0.0)||(!up&&test<0.0))
			stopScroll();
		else
			out=(int)test;

	}else{
		out=mouseNewYPos-lastMouseYPos;
	}
	lastMouseYPos=mouseNewYPos;
	lastScroll=tick;
	LeaveCriticalSection(&MouseCtrl::mouseUpdate);
	return out;
};


int MouseCtrl::getMouseDX(){
	int out=0;
	EnterCriticalSection(&MouseCtrl::mouseUpdate);
	DWORD tick= GetTickCount();
	double timeFrame=(double)(tick-lastScroll);
	if(timeFrame<2){
		LeaveCriticalSection(&MouseCtrl::mouseUpdate);
		return 0;
	}

	out=mouseNewXPos-lastMouseXPos;
	lastMouseXPos=mouseNewXPos;
	LeaveCriticalSection(&MouseCtrl::mouseUpdate);
	return out;
};

double MouseCtrl::getMouseDXMean(){
	EnterCriticalSection(&MouseCtrl::mouseUpdate);
	double speedX=(2.0*userXVel[(idPos-1)%NB_MOUSE_HISTORY]+userXVel[(idPos-2)%NB_MOUSE_HISTORY])/300.0;
	LeaveCriticalSection(&MouseCtrl::mouseUpdate);
	return speedX;
};




int MouseCtrl::updateElement(){
	int out;
	EnterCriticalSection(&MouseCtrl::mouseUpdate);
	if(this->gestMouse!=NULL)
		out=this->gestMouse->mouseGesture();
	LeaveCriticalSection(&MouseCtrl::mouseUpdate);
	return out;
};

bool MouseCtrl::isSelected(){
	return clickSelected;
};
bool MouseCtrl::isClicked(){
	if(hasInfoAboutClick)
		return false;
	else{
		hasInfoAboutClick=true;
		return splClick;
	}
};
bool MouseCtrl::isMouseDown(){
	return mouseDown;
};

bool MouseCtrl::isDblClicked(){
	return dblClick;
};

bool MouseCtrl::isSlidingLeft(){
	return slideLeft;
};
bool MouseCtrl::sthgHasChange(){
	return somethingAppend||scrolling;
};

void MouseCtrl::getMousePosition(int &x,int &y){
	if(!clickPosGet){
		x=mouseClickXPos;
		y=mouseClickYPos;
		clickPosGet=true;
	}else{
		x=mouseNewXPos;
		y=mouseNewYPos;
	}
};